fl

{
Drawing graph about offset drift of angle-velocity sensor

PropForth 5.5
2015/04/04 13:30:58

                 Prop plug
                -------------
P26(Tx) -------|RX           |
P27(Rx) -------|TX           |-----------------------PC COM5 (Processing)
               |RES          | USB
GND -----------|Vss          |
                -------------

}

\ Re-defined Word"seral" because it has bugs.
: serial
	4*
	clkfreq swap u/ dup 2/ 2/
\
\ serial structure
\
\
\ init 1st 4 members to hFF
\
	hFF h1C2 
	2dup COG!
	1+ 2dup COG!
	1+ 2dup COG!
	1+ tuck COG!
\
\ next 2 members to h100
\
	1+ h100 swap 2dup COG!
	1+ tuck COG!
\
\ bittick/4, bitticks
\
	1+ tuck COG!
	1+ tuck COG!
\
\ rxmask txmask
\
	1+ swap >m over COG!
	1+ swap >m over COG!
\ rest of structure to 0
	1+ h1F0 swap
	do
		0 i COG!
	loop
\
	c" SERIAL" numpad ccopy numpad cds W!
	4 state andnC!
\	0 io hC4 + L!    <-- always 0 cogn sersetbreak
\	0 io hC8 + L!    <-- always 0 cogn sersetflags
	_serial
;

\ =========================================================================== 
\ Constants 
\ =========================================================================== 
\ --- Jiro sensor ---
\ Slave addres h6B for L3GD20 SA0=1
hD6 wconstant L3GD20

\ register name
h20 wconstant ctrl1
h22 wconstant ctrl3
h23 wconstant ctrl4
h27 wconstant status
h28 wconstant out_x
h2A wconstant out_y

\ =========================================================================== 
\ Variables 
\ =========================================================================== 
\ --- Jyro sensor ---
variable cx
variable cy
variable cz
variable ave_x
variable ave_y
variable ave_z

wvariable inchar h100 inchar W!

\ =========================================================================== 
\ Main
\ =========================================================================== 
\ --- Jyro sensor -----------------------------------------------------------------------
\ Convert 2's complement form(16bit) to 32bit
\ None if positive
\ ( n1 -- n2 ) n1:2's complement form(16bit)  n2:32bit
: conv_32 dup h8000 and if hFFFF0000 or then ;

\ Wait until data is ready
\ ( -- )          
: dataReady?
begin
     1 status L3GD20 i2c_rd_multi err?  \ Read status register 
     8 and 0=                           \ Check bit3
     if 0 else 1 then
until                                   
;

\ Get x,y,z
\ ( -- x y z )
: get_xyz
\ Get 6byte  
6 out_x h80 or                               \ In case of multi_read, register set bit7 to 1
L3GD20 i2c_rd_multi err?                     \ Get x,y,z     
8 lshift or conv_32 >r                       \ z
8 lshift or conv_32 >r                       \ y
8 lshift or conv_32                          \ x
r> r>
;

\ Add n1 to the long contents of address
\ ( n1 addr -- )
: L+! dup L@ rot + swap L! ;

\ Get average data for 100 times
\ ( -- x y z )
: average
0 ave_x L! 0 ave_y L! 0 ave_z L!
d100 0 do
     dataReady?
     get_xyz                    
     ave_z L+! ave_y L+! ave_x L+!
loop
ave_x L@ d100 / 
ave_y L@ d100 / 
ave_z L@ d100 / 
;

\ Get offset for x,y
\ ( -- )
: offset
average 
cz L! cy L! cx L!
;

\ Get x-axis value
\ ( -- x )
: get_x
\ Get 2byte  
2 out_x h80 or 
L3GD20 i2c_rd_multi err?                   \ Get x  
8 lshift or conv_32                          \ x
;

\ Get average for x-axis for 100 times
\ ( -- x )
: average_x
0 ave_x L!
d20 0 do
     dataReady?
     get_x                    
     ave_x L+!
loop
ave_x L@ d20 / 
;

\ Get offset for x-axis
\ ( -- )
: offset_x
average_x cx L!
;

\ Show x,y,z on degree (Default:250dps)
\ ( -- )     
: disp_xyz
\ init L3GD20
8 1 ctrl3 L3GD20 i2c_wr_multi err?            
h80 1 ctrl4 L3GD20 i2c_wr_multi err?
h1F 1 ctrl1 L3GD20 i2c_wr_multi err? 
d500 delms                                   \ Wait until stable
offset

." Z:" 9 emit ." Y:" 9 emit  ." X:"  cr
0
begin
     \ Raw data below
     average     
     cz L@ - . 9 emit cy L@ - . 9 emit cx L@ - . 

     cr
     1+ dup d20 =
     if
          cr  ." Z:" 9 emit ." Y:" 9 emit  ." X:"  cr
          drop 0
     then          
     fkey? swap drop
     d100 delms 
until
drop
;
\           wvariable inchar h100 inchar W!

\ Pretreatment for communication
: pre
\ Initialize serial
\    if this bit is 0, CR is transmitted as CR LF
\    if this bit is 1, CR is transmitted as CR
1 5 sersetflags
\ pin 26 tx, pin 27 rx  d9600/4 = d2400
c" 26 27 d2400 serial" 5 cogx
d10 delms
5 cogio 2+ W@                           \ Put output ptr of cog5 on stack
inchar 5 cogio 2+ W!                    \ Set output ptr of cog5 to inchar
io 2+ W@                                \ Put output ptr of current cog on stack
5 cogio io 2+ W!                        \ Set output ptr of current cog to input ptr of cog 5
;

\ Post-processing for communication
\ ( -- )
: post
\ Restore output ptr for current cog 
io 2+ W!
\ Restore output ptr for cog 5
5 cogio 2+ W!   
\ Read out until read-buffer is empty
begin 
     inchar W@ h100 =
     if
          1
     else
          h100 inchar W! 0
     then
until     
5 cogreset
;


\ Send data for x-axis 0.5sec by 0.5sec 
\ ( -- )
: offset_drift

\ init L3GD20
8 1 ctrl3 L3GD20 i2c_wr_multi err?            
h80 1 ctrl4 L3GD20 i2c_wr_multi err?
h1F 1 ctrl1 L3GD20 i2c_wr_multi err? 
d500 delms                                   \ Wait until stable
offset_x

pre

\ Wait until receiving 'd65'
begin inchar W@ d65 = until

h100 inchar W!
40000000 cnt COG@ +
begin
     \ Check char from processing
     inchar W@ d66 <>
     if 
          h100 inchar W!
          \ Raw data below
          average_x
          cx L@ - 
          dup abs dup                         
          hFF and emit        \ Lower byte
          8 rshift emit       \ Upper byte
          \ Send sign
          0 < if 1 else 0 then emit 
          40000000 waitcnt
          0
     else
          1 
     then
until
drop

post
;
